



rosdistro_build_cache --help
IF %ERRORLEVEL% NEQ 0 exit /B 1
rosdistro_freeze_source --help
IF %ERRORLEVEL% NEQ 0 exit /B 1
rosdistro_migrate_to_rep_141 --help
IF %ERRORLEVEL% NEQ 0 exit /B 1
rosdistro_migrate_to_rep_143 --help
IF %ERRORLEVEL% NEQ 0 exit /B 1
rosdistro_reformat --help
IF %ERRORLEVEL% NEQ 0 exit /B 1
exit /B 0
